@@ -36,6 +36,7 @@
 #endif /* HAVE_UNISTD_H */
 
 #include <glib/gi18n.h>
+#include <dbus/dbus-glib.h>
 #include <gtk/gtk.h>
 
 #include "gpm-common.h"
@@ -50,22 +51,23 @@
 static void	gpm_brightness_class_init (GpmBrightnessClass *klass);
 static void	gpm_brightness_init	  (GpmBrightness      *brightness);
 static void	gpm_brightness_finalize	  (GObject	      *object);
-static gboolean	gpm_brightness_update_hw (GpmBrightness *brightness);
+static gboolean	gpm_brightness_update_hw  (GpmBrightness      *brightness);
 static int	gpm_brightness_hw_to_percent (int hw, int levels);
 
 #define GPM_BRIGHTNESS_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GPM_TYPE_BRIGHTNESS, GpmBrightnessPrivate))
 
 struct GpmBrightnessPrivate
 {
-	gboolean    has_hardware;
-	gboolean    does_own_updates;	/* keys are hardwired */
-	gboolean    does_own_dimming;	/* hardware auto-fades */
-	int	    current_hw;		/* hardware */
-	int	    level_dim_hw;
-	int	    level_std_hw;
-	int	    levels;
-	char	   *udi;
-	DBusGProxy *proxy;
+	gboolean		 has_hardware;
+	gboolean		 does_own_updates;	/* keys are hardwired */
+	gboolean		 does_own_dimming;	/* hardware auto-fades */
+	int			 current_hw;		/* hardware */
+	int			 level_dim_hw;
+	int			 level_std_hw;
+	int			 levels;
+	char			*udi;
+	DBusGProxy		*proxy;
+	GpmHal			*hal;
 };
 
 enum {
@@ -131,11 +133,14 @@ gpm_brightness_init (GpmBrightness *brightness)
 	gchar  **names;
 	char    *manufacturer_string = NULL;
 	gboolean res;
+	GError  *error = NULL;
 
 	brightness->priv = GPM_BRIGHTNESS_GET_PRIVATE (brightness);
 
+	brightness->priv->hal = gpm_hal_new ();
+
 	/* save udi of lcd adapter */
-	gpm_hal_find_device_capability ("laptop_panel", &names);
+	gpm_hal_device_find_capability (brightness->priv->hal, "laptop_panel", &names);
 	if (names == NULL || names[0] == NULL) {
 		brightness->priv->has_hardware = FALSE;
 		gpm_debug ("No devices of capability laptop_panel");
@@ -144,7 +149,7 @@ gpm_brightness_init (GpmBrightness *brightness)
 
 	/* We only want first laptop_panel object (should only be one) */
 	brightness->priv->udi = g_strdup (names[0]);
-	gpm_hal_free_capability (names);
+	gpm_hal_free_capability (brightness->priv->hal, names);
 
 	brightness->priv->has_hardware = TRUE;
 	brightness->priv->does_own_dimming = FALSE;
@@ -153,7 +158,7 @@ gpm_brightness_init (GpmBrightness *brightness)
 	 * and don't do the new-fangled dimming routine. The ThinkPad dims
 	 * gently itself and the two dimming routines just get messy.
 	 * https://bugzilla.redhat.com/bugzilla/show_bug.cgi?id=173382 */
-	gpm_hal_device_get_string (HAL_ROOT_COMPUTER,
+	gpm_hal_device_get_string (brightness->priv->hal, HAL_ROOT_COMPUTER,
 				   "smbios.system.manufacturer",
 				   &manufacturer_string);
 	if (manufacturer_string) {
@@ -167,7 +172,7 @@ gpm_brightness_init (GpmBrightness *brightness)
 	/* We only want to change the brightness if the machine does not
 	   do it on it's own updates, as this can make the panel flash in a
 	   feedback loop. */
-	res = gpm_hal_device_get_bool (brightness->priv->udi,
+	res = gpm_hal_device_get_bool (brightness->priv->hal, brightness->priv->udi,
 				       "laptop_panel.brightness_in_hardware",
 				       &brightness->priv->does_own_updates);
 	/* This key does not exist on normal machines */
@@ -176,14 +181,19 @@ gpm_brightness_init (GpmBrightness *brightness)
 	}
 
 	/* get proxy once and store */
-	gpm_hal_get_dbus_connection (&system_connection);
+	system_connection = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
+	if (error) {
+		gpm_warning ("%s", error->message);
+		g_error_free (error);
+		return;
+	}
 	brightness->priv->proxy = dbus_g_proxy_new_for_name (system_connection,
 							     HAL_DBUS_SERVICE,
 							     brightness->priv->udi,
 							     HAL_DBUS_INTERFACE_LAPTOP_PANEL);
 
 	/* get levels that the adapter supports -- this does not change ever */
-	gpm_hal_device_get_int (brightness->priv->udi, "laptop_panel.num_levels",
+	gpm_hal_device_get_int (brightness->priv->hal, brightness->priv->udi, "laptop_panel.num_levels",
 				&brightness->priv->levels);
 
 	/* this changes under our feet */
@@ -210,6 +220,7 @@ gpm_brightness_finalize (GObject *object)
 
 	g_free (brightness->priv->udi);
 	g_object_unref (G_OBJECT (brightness->priv->proxy));
+	g_object_unref (brightness->priv->hal);
 
 	g_return_if_fail (brightness->priv != NULL);
 	G_OBJECT_CLASS (gpm_brightness_parent_class)->finalize (object);
@@ -242,6 +253,8 @@ gpm_brightness_update_hw (GpmBrightness *brightness)
 	gboolean retval;
 	gint     brightness_hw = 0;
 
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
+
 	retval = TRUE;
 	if (!dbus_g_proxy_call (brightness->priv->proxy, "GetBrightness", &error,
 				G_TYPE_INVALID,
@@ -268,12 +281,14 @@ gpm_brightness_update_hw (GpmBrightness *brightness)
  **/
 static gboolean
 gpm_brightness_set_hw (GpmBrightness *brightness,
-			     int	    brightness_level_hw)
+		       int	      brightness_level_hw)
 {
 	GError  *error = NULL;
 	gint     ret;
 	gboolean retval;
 
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
+
 	if (brightness_level_hw < 0 ||
 	    brightness_level_hw > brightness->priv->levels - 1) {
 		gpm_warning ("set outside range (%i of %i)",
@@ -329,35 +344,87 @@ gpm_brightness_percent_to_hw (int percentage,
  * gpm_brightness_dim_hw:
  * @brightness: This brightness class instance
  * @new_level_hw: The new hardware level
+ *
+ * Just do the step up and down, after knowing the step interval
  **/
 static void
-gpm_brightness_dim_hw (GpmBrightness *brightness,
-			     int	    new_level_hw)
+gpm_brightness_dim_hw_step (GpmBrightness *brightness,
+		            int            new_level_hw,
+		            int		   step_interval)
 {
-	int   current_hw;
-	int   a;
-
-	if (! brightness->priv->has_hardware) {
-		return;
-	}
+	int current_hw;
+	int a;
 
+	g_return_if_fail (GPM_IS_BRIGHTNESS (brightness));
 	current_hw = brightness->priv->current_hw;
 
+	/* we do the step interval as we can have insane levels of brightness */
 	if (new_level_hw > current_hw) {
 		/* going up */
-		for (a=current_hw; a <= new_level_hw; a++) {
+		for (a=current_hw; a <= new_level_hw; a+=step_interval) {
 			gpm_brightness_set_hw (brightness, a);
 			g_usleep (1000 * DIM_INTERVAL);
 		}
 	} else {
 		/* going down */
-		for (a=current_hw; a >= new_level_hw; a--) {
+		for (a=current_hw; a >= new_level_hw; a-=step_interval) {
 			gpm_brightness_set_hw (brightness, a);
 			g_usleep (1000 * DIM_INTERVAL);
 		}
 	}
 }
 
+/**
+ * gpm_brightness_get_step:
+ * @brightness: This brightness class instance
+ * Return value: the amount of hardware steps to do on each update or
+ * zero for error.
+ **/
+static int
+gpm_brightness_get_step (GpmBrightness *brightness)
+{
+	int step;
+
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), 0);
+
+	if (brightness->priv->levels < 20) {
+		/* less than 20 states should do every state */
+		step = 1;
+	} else {
+		/* macbook pro has a bazzillion brightness levels, do in 5% steps */
+		step = brightness->priv->levels / 20;
+	}
+	return step;
+}
+
+/**
+ * gpm_brightness_dim_hw:
+ * @brightness: This brightness class instance
+ * @new_level_hw: The new hardware level
+ **/
+static void
+gpm_brightness_dim_hw (GpmBrightness *brightness,
+		       int	      new_level_hw)
+{
+	int step;
+
+	g_return_if_fail (GPM_IS_BRIGHTNESS (brightness));
+
+	if (! brightness->priv->has_hardware) {
+		return;
+	}
+
+	/* some machines don't take kindly to auto-dimming */
+	if (brightness->priv->does_own_dimming) {
+		gpm_brightness_set_hw (brightness, new_level_hw);
+		return;
+	}
+
+	/* macbook pro has a bazzillion brightness levels, be a bit clever */
+	step = gpm_brightness_get_step (brightness);
+	gpm_brightness_dim_hw_step (brightness, new_level_hw, step);
+}
+
 /**
  * gpm_brightness_hw_to_percent:
  * @hw: The hardware level
@@ -414,7 +481,8 @@ gpm_brightness_set_level_std (GpmBrightness *brightness,
 			      int	     brightness_level)
 {
 	int level_hw;
-	level_hw = gpm_brightness_percent_to_hw (brightness_level, brightness->priv->levels);
+	level_hw = gpm_brightness_percent_to_hw (brightness_level,
+						 brightness->priv->levels);
 	brightness->priv->level_std_hw = level_hw;
 }
 
@@ -449,13 +517,13 @@ gpm_brightness_undim (GpmBrightness *brightness)
 void
 gpm_brightness_set (GpmBrightness *brightness)
 {
-	gpm_brightness_set_hw (brightness, brightness->priv->level_std_hw);
+	gpm_brightness_dim_hw (brightness, brightness->priv->level_std_hw);
 }
 
 /**
  * gpm_brightness_get:
  * @brightness: This brightness class instance
- * Return value: The percentage brightness, or -1 for no hardware
+ * Return value: The percentage brightness, or -1 for no hardware or error
  *
  * Gets the current (or at least what this class thinks is current) percentage
  * brightness. This is quick as no HAL inquiry is done.
@@ -464,6 +532,9 @@ int
 gpm_brightness_get (GpmBrightness *brightness)
 {
 	int percentage;
+
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), -1);
+
 	if (! brightness->priv->has_hardware) {
 		return -1;
 	}
@@ -481,6 +552,11 @@ gpm_brightness_get (GpmBrightness *brightness)
 void
 gpm_brightness_up (GpmBrightness *brightness)
 {
+	int step;
+	int percentage;
+
+	g_return_if_fail (GPM_IS_BRIGHTNESS (brightness));
+
 	if (! brightness->priv->has_hardware) {
 		return;
 	}
@@ -489,10 +565,11 @@ gpm_brightness_up (GpmBrightness *brightness)
 	if (brightness->priv->does_own_updates) {
 		gpm_brightness_update_hw (brightness);
 	} else {
-		gpm_brightness_set_hw (brightness, brightness->priv->current_hw + 1);
+		/* macbook pro has a bazzillion brightness levels, be a bit clever */
+		step = gpm_brightness_get_step (brightness);
+		gpm_brightness_set_hw (brightness, brightness->priv->current_hw + step);
 	}
 
-	int percentage;
 	percentage = gpm_brightness_hw_to_percent (brightness->priv->current_hw,
 						   brightness->priv->levels);
 	gpm_debug ("emitting brightness-step-changed : %i", percentage);
@@ -508,6 +585,11 @@ gpm_brightness_up (GpmBrightness *brightness)
 void
 gpm_brightness_down (GpmBrightness *brightness)
 {
+	int step;
+	int percentage;
+
+	g_return_if_fail (GPM_IS_BRIGHTNESS (brightness));
+
 	if (! brightness->priv->has_hardware) {
 		return;
 	}
@@ -516,12 +598,28 @@ gpm_brightness_down (GpmBrightness *brightness)
 	if (brightness->priv->does_own_updates) {
 		gpm_brightness_update_hw (brightness);
 	} else {
+		/* macbook pro has a bazzillion brightness levels, be a bit clever */
+		step = gpm_brightness_get_step (brightness);
 		gpm_brightness_set_hw (brightness, brightness->priv->current_hw - 1);
 	}
 
-	int percentage;
 	percentage = gpm_brightness_hw_to_percent (brightness->priv->current_hw,
 						   brightness->priv->levels);
 	gpm_debug ("emitting brightness-step-changed : %i", percentage);
 	g_signal_emit (brightness, signals [BRIGHTNESS_STEP_CHANGED], 0, percentage);
 }
+
+/**
+ * gpm_brightness_has_hardware:
+ * @brightness: This brightness class instance
+ * Return value: if we can set or get the lcd brightness
+ *
+ * Returns if the computer has brightness hardware (lcd panel)
+ **/
+gboolean
+gpm_brightness_has_hardware (GpmBrightness *brightness)
+{
+	g_return_val_if_fail (GPM_IS_BRIGHTNESS (brightness), FALSE);
+
+	return brightness->priv->has_hardware;
+}
