@@ -23,16 +23,29 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <gdk/gdk.h>
 
 #include "gpm-common.h"
 
-/** Returns the time string, e.g. "2 hours 3 minutes"
+/**
+ * gpm_warning_beep
+ **/
+void
+gpm_warning_beep (void)
+{
+	GdkDisplay *display = gdk_display_get_default ();
+	gdk_display_beep (display);
+}
+
+/**
+ * gpm_get_timestring:
+ * @time: The time value to convert in seconds
+ * @cookie: The cookie we are looking for
  *
- *  @param  time		Time to convert to string (in seconds)
- *  @return			The timestring
+ * Returns a localised timestring
  *
- *  @note	time == 0 is returned as "Unknown"
- */
+ * Return value: The time string, e.g. "2 hours 3 minutes"
+ **/
 char *
 gpm_get_timestring (int time)
 {
@@ -50,8 +63,8 @@ gpm_get_timestring (int time)
 
 	if (minutes < 60) {
 		timestring = g_strdup_printf (ngettext ("%i minute",
-                                                        "%i minutes",
-                                                        minutes), minutes);
+							"%i minutes",
+							minutes), minutes);
 		return timestring;
 	}
 
