@@ -26,6 +26,7 @@
 #include <glade/glade.h>
 #include <libgnomeui/gnome-help.h>
 #include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include <string.h>
 #include "gpm-inhibit.h"
 #include "gpm-debug.h"
@@ -136,6 +137,8 @@ gpm_inhibit_add (GpmInhibit *inhibit,
 		 const char *application,
 		 const char *reason)
 {
+	GpmInhibitData *data = g_new (GpmInhibitData, 1);
+
 	/* handle where the application does not add required data */
 	if (connection == NULL ||
 	    application == NULL ||
@@ -146,7 +149,6 @@ gpm_inhibit_add (GpmInhibit *inhibit,
 	}
 
 	/* seems okay, add to list */
-	GpmInhibitData *data = g_new (GpmInhibitData, 1);
 	data->cookie = gpm_inhibit_generate_cookie (inhibit);
 	data->application = g_strdup (application);
 	data->connection = g_strdup (connection);
@@ -287,16 +289,16 @@ gpm_inhibit_get_message (GpmInhibit *inhibit,
 
 	if (g_slist_length (inhibit->priv->list) == 1) {
 		data = (GpmInhibitData *) g_slist_nth_data (inhibit->priv->list, 0);
-		g_string_append_printf (message, "<b>%s</b> has stopped the "
-					"%s from taking place because <i>%s</i>.",
+		g_string_append_printf (message, _("<b>%s</b> has stopped the "
+					"%s from taking place : <i>%s</i>."),
 					data->application, action, data->reason);
 	} else {
-		g_string_append_printf (message, "Multiple applications have stopped the "
-					"%s from taking place:", action);
+		g_string_append_printf (message, _("Multiple applications have stopped the "
+					"%s from taking place."), action);
 		for (a=0; a<g_slist_length (inhibit->priv->list); a++) {
 			data = (GpmInhibitData *) g_slist_nth_data (inhibit->priv->list, a);
 			g_string_append_printf (message,
-						"\n<b>%s</b> because <i>%s</i>",
+						"\n<b>%s</b> : <i>%s</i>",
 						data->application, data->reason);
 		}
 	}
@@ -327,6 +329,9 @@ static void
 gpm_inhibit_finalize (GObject *object)
 {
 	GpmInhibit *inhibit;
+	int a;
+	GpmInhibitData *data;
+
 	g_return_if_fail (object != NULL);
 	g_return_if_fail (GPM_IS_INHIBIT (object));
 
@@ -334,8 +339,6 @@ gpm_inhibit_finalize (GObject *object)
 	inhibit->priv = GPM_INHIBIT_GET_PRIVATE (inhibit);
 
 	/* remove items in list and free */
-	int a;
-	GpmInhibitData *data;
 	for (a=0; a<g_slist_length (inhibit->priv->list); a++) {
 		data = (GpmInhibitData *) g_slist_nth_data (inhibit->priv->list, a);
 		gpm_inhibit_free_data_object (data);
